package com.llisoft.sms.config;

import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.NoHandlerFoundException;

import com.llisoft.sms.vo.CodeEnum;
import com.llisoft.sms.vo.ResponseVo;

/**
 * 统一处理业务异常 controller
 * 只能拦截到controller层的异常
 * 如需拦截404等系统级异常，需要有如下配置
 * spring.resources.addMappings: false #不为工程中的资源文件建立映射
 * spring.mvc.throwExceptionIfNoHandlerFound: true #出现异常时直接抛出
 * @author YangJie [2018年12月26日]
 */
@RestControllerAdvice
public class ExceptionHandlerConfig {
	
	private Logger logger = LoggerFactory.getLogger(ExceptionHandler.class);
	

    /**
     * 入参空
     * @param exception
     * @return
     */
    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseVo<String> exception(HttpMessageNotReadableException exception){
    	logger.warn("入参异常: {}", exception.getMessage());
        return ResponseVo.build(CodeEnum.C400);
    }
    
    /**
     * 入参检验失败
     * @param exception
     * @return
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseVo<String> exception(MethodArgumentNotValidException exception){
    	logger.warn("入参异常: {}", exception.getMessage());
    	return ResponseVo.build(CodeEnum.C400.getCode(), exception.getBindingResult().getFieldError().getDefaultMessage());
    }
    
    /**
     * 业务异常
     * @param exception
     * @return
     */
    @ExceptionHandler(MtaException.class)
    public ResponseVo<String> exception(MtaException exception){
    	CodeEnum codeEnum = exception.getCodeEnum();
    	if (Objects.nonNull(codeEnum)) {
    		return ResponseVo.build(codeEnum.getCode(), codeEnum.getMsg());
		}
    	logger.warn("业务异常: {}", exception.getMessage());
    	return ResponseVo.build(CodeEnum.FAIL.getCode(), exception.getMessage());
    }

    
    /**
     * 地址异常
     * @param exception
     * @return
     */
    @ExceptionHandler(NoHandlerFoundException.class)
    public ResponseVo<String> exception(NoHandlerFoundException exception){
    	logger.warn("地址异常: {}", exception.getMessage());
    	return ResponseVo.build(CodeEnum.C404);
    }
    
    /**
     * 默认异常
     * @param exception
     * @return
     */
    @ExceptionHandler(Exception.class)
    public ResponseVo<String> exception(Exception exception){
    	logger.error("服务异常", exception);
    	return ResponseVo.build(CodeEnum.C500);
    }
    
}
